@@ -1,6 +1,8 @@
 #ifndef HWTABLE_H
 #define HWTABLE_H
+
 #define CHECKINT 5
+#define MULTIPATH "/sbin/multipath -q -S"
 
 #include "checkers.h"
 #include "vector.h"
@@ -26,5 +28,6 @@ vector hwtable;
 vector blist;
 vector binvec;
 int checkint;
+char * multipath;
 
 #endif
